import time
import os
import datetime

import exceptions

from base import (
    StorageServiceBase,
    FileStat,
    )


class IOErrorContext(object):
    def __enter__(self):
        return self

    def __exit__(self, type_, value, traceback):
        if type_ is IOError and value:
            errno, errstr = value.args
            for exc in exceptions.ClosetError.__subclasses__():
                if exc.fs_errno == errno:
                    raise exc(errstr)


class FilesystemFileStat(FileStat):
    def __init__(self, statinfo):
        # Extract the raw data from stat
        args = {
            'size': statinfo.st_size,
            'atime': statinfo.st_atime,
            'mtime': statinfo.st_mtime,
            'ctime': statinfo.st_ctime,
        }

        # If birthtime exists, use it
        try:
            args['ctime'] = statinfo.st_birthtime if statinfo.st_birthtime is not None else args['ctime']
        except AttributeError:
            pass

        # Fix times
        for k in ('atime', 'mtime', 'ctime'):
            if args[k] is not None:
                args[k] = time.mktime(datetime.datetime.utcfromtimestamp(int(args[k])).timetuple())

        super(FilesystemFileStat, self).__init__(**args)


class FilesystemStorage(StorageServiceBase):
    def __init__(self, name=None, bucket=None, base_directory=None, *args, **kwargs):
        super(FilesystemStorage, self).__init__(name=name, bucket=bucket, *args, **kwargs)
        self.base_directory = base_directory or '/tmp'

    def _normalize_path(self, filename, external=False):
        filename = super(FilesystemStorage, self)._normalize_path(filename)
        if not external:
            filename = os.path.join(self.base_directory, filename)
            if not filename.startswith(self.base_directory):
                raise exceptions.FileOutsideBaseDirectoryError(filename)
        return filename

    def exists(self, filename):
        return os.path.exists(self._normalize_path(filename))

    def open(self, filename, mode=None):
        mode = mode or 'rb'
        filename = self._normalize_path(filename)
        with IOErrorContext():
            if mode[0] in ('w', 'a') or mode[-1] == '+':
                destination_directory = os.path.dirname(filename)
                if not os.path.exists(destination_directory):
                    os.makedirs(destination_directory)

            return open(filename, mode)

    def stat(self, filename):
        with IOErrorContext():
            return FilesystemFileStat(os.stat(self._normalize_path(filename)))

    def delete(self, filename):
        with IOErrorContext():
            return os.unlink(self._normalize_path(filename))



import os

import magic

from exceptions import (
    FileNotFoundError,
    NoBaseURLError,
    )


class StorageServiceBase(object):
    def __init__(self, name=None, bucket=None, base_url=None, *args, **kwargs):
        self.name = name or 'default'
        self.bucket = bucket or (self.name if bucket is None else None)
        self.base_url = base_url

    def _normalize_path(self, filename, external=False):
        if self.bucket:
            filename = os.path.join(self.bucket, filename)
        if external and self.base_url:
            filename = os.path.join(self.base_url, filename)
        filename = os.path.normpath(filename)
        return filename

    def url_for(self, filename):
        if not self.base_url:
            raise NoBaseURLError("No base URL was provided")

        return self.base_url + '/' + self._normalize_path(filename, external=True)

    def exists(self, filename):
        raise NotImplementedError()

    def open(self, filename, mode=None):
        raise NotImplementedError()

    def stat(self, filename):
        raise NotImplementedError()

    def delete(self, filename):
        raise NotImplementedError()

    def mimetype(self, filename):
        return self.open(filename, mode='rb').mimetype

    @staticmethod
    def mimetype_for(file_obj):
        if not hasattr(file_obj, '_mimetype'):
            pos = file_obj.tell()
            file_obj.seek(0)
            file_obj._mimetype = magic.from_buffer(file_obj.read(1024), mime=True)
            file_obj.seek(pos)
        return file_obj._mimetype


class FileStat(object):
    def __init__(self, size, atime, mtime, ctime):
        self.size = size
        self.atime = atime
        self.mtime = mtime
        self.ctime = ctime



import os

from setuptools import setup

# Using vbox, hard links do not work
if os.environ.get('USER','') == 'vagrant':
    del os.link

setup(
    name='closet',
    version='0.1.5',
    description='File storage abstraction',
    long_description='',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: Other/Proprietary License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
    ],
    url='https://github.com/BasementCat/closet',
    author='Alec Elton',
    author_email='alec.elton@gmail.com',
    license='',
    packages=['closet'],
    install_requires=['python-magic'],
    test_suite='nose.collector',
    tests_require=['nose'],
    zip_safe=False
)


import errno


class ClosetError(Exception):
    fs_errno = None


class FileOutsideBaseDirectoryError(ClosetError):
    pass


class NoBaseURLError(ClosetError):
    pass


class FileNotFoundError(ClosetError):
    fs_errno = errno.ENOENT


class AccessDeniedError(ClosetError):
    fs_errno = errno.EACCES


class NotADirectoryError(ClosetError):
    fs_errno = errno.ENOTDIR


class IsADirectoryError(ClosetError):
    fs_errno = errno.EISDIR


class NoSpaceError(ClosetError):
    fs_errno = errno.ENOSPC


class ReadOnlyError(ClosetError):
    fs_errno = errno.EROFS
